@@ -238,14 +238,14 @@ public:
 
     /** Select the move to play.
         Uses select_child_final() on the root node. */
-    bool select_move(Move& mv, vector<Move>* exclude_moves = 0) const;
+    bool select_move(Move& mv, const vector<Move>* exclude_moves = 0) const;
 
     /** Select the best child of a node after the search.
         Selects child with highest visit count; the value is used as a
         tie-breaker for equal counts (important at very low number of
         simulations, e.g. all children have count 1 or 0). */
     const Node* select_child_final(const Node& node,
-                                   vector<Move>* exclude_moves = 0) const;
+                                   const vector<Move>* exclude_moves = 0) const;
 
     State& get_state();
 
@@ -797,6 +797,11 @@ bool Search<S, M, P>::search(Move& mv, ValueType max_count,
                              size_t min_simulations, double max_time,
                              TimeSource& time_source, bool always_search)
 {
+    if (max_count > 0)
+        // A fixed number of simulations means that no time limit is used, but
+        // max_time is still used at some places in the code, so we set it to
+        // infinity
+        max_time = numeric_limits<double>::max();
     bool clear_tree = true;
     bool is_followup = check_followup(m_followup_sequence);
     if (is_followup && get_reuse_param() == get_last_reuse_param())
@@ -815,6 +820,11 @@ bool Search<S, M, P>::search(Move& mv, ValueType max_count,
             const Node* node = find_node(m_tree, m_followup_sequence);
             if (node != 0)
             {
+                // The values of root nodes have a different meaning than the
+                // values of inner nodes (position value vs. move values) so
+                // we might have to discard them
+                if (node != &m_tree.get_root())
+                    m_tree.clear_values(*node);
                 TimeIntervalChecker interval_checker(time_source, max_time);
                 bool aborted = ! m_tree.extract_subtree(m_tmp_tree, *node, true,
                                                         &interval_checker);
@@ -931,15 +941,20 @@ const Node<M>* Search<S, M, P>::select_child(const Node& node)
     ValueType best_explored_value = -numeric_limits<ValueType>::max();
     ValueType best_unexplored_value = -numeric_limits<ValueType>::max();
     ValueType bias_term_constant_part = 0; // Init to avoid compiler warning
+    // Note: use visit count here not count. In most cases, count is larger
+    // than visit count because of prior knowledge initializaion, but it can
+    // happen that count is zero and visit count greater zero if the node
+    // value was cleared but not the visit count after reusing an inner node
+    // from a previous search as the root of the new search
+    ValueType node_count = node.get_visit_count();
     if (m_bias_term_constant != 0)
     {
-        ValueType count = node.get_count();
-        LIBBOARDGAME_ASSERT(count > 0);
+        LIBBOARDGAME_ASSERT(node_count > 0);
         bias_term_constant_part =
-            m_bias_term_constant_sq * m_fast_log.get_log(float(count));
+            m_bias_term_constant_sq * m_fast_log.get_log(float(node_count));
     }
-    ValueType beta = sqrt(m_rave_equivalence
-                          / (3 * node.get_count() + m_rave_equivalence));
+    ValueType beta =
+        sqrt(m_rave_equivalence / (3 * node_count + m_rave_equivalence));
     if (log_move_selection)
         log() << "beta=" << beta << '\n';
     unsigned int nu_explored = 0;
@@ -1029,17 +1044,18 @@ const Node<M>* Search<S, M, P>::select_child(const Node& node)
 
 template<class S, class M, unsigned int P>
 const Node<M>* Search<S, M, P>::select_child_final(const Node& node,
-                                             vector<Move>* exclude_moves) const
+                                        const vector<Move>* exclude_moves) const
 {
+    // Select the child with the highest visit count, use value as a tie breaker
     const Node* result = 0;
     ValueType max_count = -1;
-    ValueType max_count_value = numeric_limits<ValueType>::max();
+    ValueType max_count_value = -numeric_limits<ValueType>::max();
     for (ChildIterator i(node); i; ++i)
     {
         ValueType count = i->get_visit_count();
         if (count > max_count
             || (count == max_count && count > 0 && max_count > 0
-                && i->get_value() < max_count_value))
+                && i->get_value() > max_count_value))
         {
             if (exclude_moves != 0
                 && find(exclude_moves->begin(), exclude_moves->end(),
@@ -1055,7 +1071,7 @@ const Node<M>* Search<S, M, P>::select_child_final(const Node& node,
 }
 
 template<class S, class M, unsigned int P>
-bool Search<S, M, P>::select_move(Move& mv, vector<Move>* exclude_moves)
+bool Search<S, M, P>::select_move(Move& mv, const vector<Move>* exclude_moves)
     const
 {
     const Node* child = select_child_final(m_tree.get_root(), exclude_moves);
