@@ -59,6 +59,7 @@ QToolButton* createOBoxToolButton(QAction* action)
     button->setDefaultAction(action);
     button->setAutoRaise(true);
     button->setFocusPolicy(Qt::NoFocus);
+    button->setIconSize(QSize(16, 16));
     return button;
 }
 
@@ -232,7 +233,7 @@ MainWindow::MainWindow(const QString& initialFile, bool noBook)
     bool showToolbar = settings.value("toolbar", true).toBool();
     m_toolBar->setVisible(showToolbar);
     initGameVariantActions();
-    setWindowIcon(QIcon(":/pentobi/pentobi.png"));
+    setWindowIcon(QIcon(":/pentobi/pentobi.svg"));
 
     if (! restoreGeometry(settings.value("geometry").toByteArray()))
         adjustSize();
@@ -1796,7 +1797,7 @@ void MainWindow::open(const QString& file, bool isTemporary)
         m_game->goto_node(get_last_node(m_game->get_root()));
         initPieceSelectors();
     }
-    catch (const InvalidPropertyValue& e)
+    catch (const Exception& e)
     {
         showInvalidFile(file, e);
     }
@@ -1987,24 +1988,35 @@ void MainWindow::save()
         saveAs();
         return;
     }
+    if (save(m_file))
+    {
+        m_game->clear_modified();
+        updateWindow(false);
+    }
+}
+
+bool MainWindow::save(const QString& file)
+{
 #ifdef VERSION
     m_game->set_application("Pentobi", VERSION);
 #else
     m_game->set_application("Pentobi");
 #endif
-    ofstream out(m_file.toLocal8Bit().constData());
+    ofstream out(file.toLocal8Bit().constData());
     write_tree(out, m_game->get_root(), true, 2);
     if (! out)
+    {
         showError(tr("The file could not be saved."),
                   /*: Error message if file cannot be saved. %1 is
                     replaced by the file name, %2 by the error message
                     of the operating system. */
-                  tr("%1: %2").arg(m_file).arg(strerror(errno)));
+                  tr("%1: %2").arg(file).arg(strerror(errno)));
+        return false;
+    }
     else
     {
-        showStatus(tr("File saved %1").arg(m_file), true);
-        m_game->clear_modified();
-        updateWindow(false);
+        showStatus(tr("File saved %1").arg(file), true);
+        return true;
     }
 }
 
@@ -2029,8 +2041,12 @@ void MainWindow::saveAs()
     file = QFileDialog::getSaveFileName(this, tr("Save"), file, getFilter());
     if (! file.isEmpty())
     {
+        if (save(file))
+        {
+            m_game->clear_modified();
+            updateWindow(false);
+        }
         setFile(file);
-        save();
     }
 }
 
@@ -2256,8 +2272,9 @@ void MainWindow::setFile(const QString& file)
         while (files.size() > maxRecentFiles)
             files.removeLast();
         settings.setValue("recent_files", files);
-        updateRecentFiles();
         settings.setValue("last_dir", info.dir().path());
+        settings.sync();
+        updateRecentFiles();
     }
 }
 
@@ -2316,7 +2333,7 @@ void MainWindow::setMoveNumberText()
     const Node* node = &current;
     do
     {
-        if (! tree.get_move(*node).is_null())
+        if (! tree.get_move_ignore_invalid(*node).is_null())
             ++move;
         node = node->get_parent_or_null();
     }
@@ -2325,7 +2342,7 @@ void MainWindow::setMoveNumberText()
     node = current.get_first_child_or_null();
     while (node != 0)
     {
-        if (! tree.get_move(*node).is_null())
+        if (! tree.get_move_ignore_invalid(*node).is_null())
             ++nuMoves;
         node = node->get_first_child_or_null();
     }
@@ -2450,9 +2467,10 @@ void MainWindow::showComment(bool checked)
     }
 }
 
-void MainWindow::showError(const QString& text, const QString& infoText)
+void MainWindow::showError(const QString& text, const QString& infoText,
+                           const QString& detailText)
 {
-    showMessage(QMessageBox::Critical, text, infoText);
+    showMessage(QMessageBox::Critical, text, infoText, detailText);
 }
 
 void MainWindow::showGameOver()
@@ -2484,7 +2502,7 @@ void MainWindow::showGameOver()
             info = tr("Blue/Red wins with 1 point.");
         else if (score > 0)
             info = tr("Blue/Red wins with %1 points.").arg(score);
-        else if (score == 1)
+        else if (score == -1)
             info = tr("Yellow/Green wins with 1 point.");
         else if (score < 0)
             info = tr("Yellow/Green wins with %1 points.").arg(-score);
@@ -2556,20 +2574,20 @@ void MainWindow::showGameOver()
     showInfo(tr("The game is over."), info);
 }
 
-void MainWindow::showInfo(const QString& text, const QString& infoText)
+void MainWindow::showInfo(const QString& text, const QString& infoText,
+                          const QString& detailText)
 {
-    showMessage(QMessageBox::NoIcon, text, infoText);
+    showMessage(QMessageBox::NoIcon, text, infoText, detailText);
 }
 
 void MainWindow::showInvalidFile(QString file, const Exception& e)
 {
     showError(tr("Error in file '%1'").arg(QFileInfo(file).fileName()),
-              tr("The file is not a valid Blokus SGF file."));
-    log() << e.what() << '\n';
+              tr("The file is not a valid Blokus SGF file."), e.what());
 }
 
 void MainWindow::showMessage(QMessageBox::Icon icon, const QString& text,
-                             const QString& infoText)
+                             const QString& infoText, const QString& detailText)
 {
     // Workaround to avoid very small widths if the main text is short, which
     // causes ugly word wrapping with single-word lines in the informative text.
@@ -2584,6 +2602,7 @@ void MainWindow::showMessage(QMessageBox::Icon icon, const QString& text,
     msgBox.setIcon(icon);
     msgBox.setText(expandedText);
     msgBox.setInformativeText(infoText);
+    msgBox.setDetailedText(detailText);
     msgBox.exec();
 }
 
@@ -2651,7 +2670,8 @@ void MainWindow::truncate()
 void MainWindow::undo()
 {
     const Node& current = m_game->get_current();
-    if (current.has_children() || ! m_game->get_tree().has_move(current))
+    if (current.has_children()
+        || ! m_game->get_tree().has_move_ignore_invalid(current))
         return;
     truncate();
 }
@@ -2684,7 +2704,7 @@ void MainWindow::updateFlipActions()
 
 void MainWindow::updateMoveAnnotationActions()
 {
-    if (m_game->get_move().is_null())
+    if (m_game->get_move_ignore_invalid().is_null())
     {
         m_menuMoveAnnotation->setEnabled(false);
         return;
@@ -2815,7 +2835,7 @@ void MainWindow::updateWindow(bool currentNodeChanged)
     bool isMain = is_main_variation(current);
     bool hasParent = current.has_parent();
     bool hasChildren = current.has_children();
-    bool hasMove = tree.has_move(current);
+    bool hasMove = tree.has_move_ignore_invalid(current);
     m_actionBackToMainVariation->setEnabled(! isMain);
     m_actionBeginning->setEnabled(hasParent);
     m_actionBackward->setEnabled(hasParent);
